import logging
from lxml import etree
import requests

from odoo import api, fields, models
from odoo.exceptions import UserError
from odoo.tools.translate import _

from odoo.addons.saas_worker.const import TIMEOUT

_logger = logging.getLogger(__name__)


def xml2json_from_elementtree(el, preserve_whitespaces=False):
    """ xml2json-direct
    Simple and straightforward XML-to-JSON converter in Python
    New BSD Licensed
    http://code.google.com/p/xml2json-direct/
    """
    res = {}
    if el.tag[0] == "{":
        ns, name = el.tag.rsplit("}", 1)
        res["tag"] = name
        res["namespace"] = ns[1:]
    else:
        res["tag"] = el.tag
    res["attrs"] = {}
    for k, v in el.items():
        res["attrs"][k] = v
    kids = []
    if el.text and (preserve_whitespaces or el.text.strip() != ''):
        kids.append(el.text)
    for kid in el:
        kids.append(xml2json_from_elementtree(kid, preserve_whitespaces))
        if kid.tail and (preserve_whitespaces or kid.tail.strip() != ''):
            kids.append(kid.tail)
    res["children"] = kids
    return res


class ResCompany(models.Model):
    _inherit = 'res.company'

    @api.model
    def run_update_currency(self):
        # from currency_rate_live/models/res_config_settings.py

        # from method res.company:_parse_ecb_data
        request_url = "http://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml"
        parse_url = requests.get(request_url, timeout=TIMEOUT)
        parse_url.raise_for_status()
        xmlstr = etree.fromstring(parse_url.content)
        data = xml2json_from_elementtree(xmlstr)
        node = data['children'][2]['children'][0]
        xmldate = fields.Date.to_date(node['attrs']['time'])

        available_currency_names = self.env['res.currency'].search([]).mapped('name')
        parsed_data = {x['attrs']['currency']: (float(x['attrs']['rate']), xmldate)
                       for x in node['children']
                       if x['attrs']['currency'] in available_currency_names}

        if parsed_data and 'EUR' in available_currency_names:
            parsed_data['EUR'] = (1.0, xmldate)

        # from method res.company:_generate_currency_rates
        # reworked the code to minimize the amount of search queries
        Currency = self.env['res.currency']
        CurrencyRate = self.env['res.currency.rate']

        currency_id_by_name = {x.name: x for x in Currency.search([('name', 'in', list(parsed_data))])}
        currency_rates = {(x.currency_id.id, x.name, x.company_id.id): x
                          for x in CurrencyRate.search([
                              ('currency_id.name', 'in', list(parsed_data)),
                              ('name', 'in', [x[1] for x in parsed_data.values()]),
                          ])}
        for company in self.env['res.company'].search([]):
            rate_info = parsed_data.get(company.currency_id.name, None)
            if not rate_info:
                msg = _("Your main currency (%s) is not supported by this exchange rate provider. Please choose another one.",
                        company.currency_id.name)
                if self._context.get('suppress_errors'):
                    _logger.warning(msg)
                    continue
                else:
                    raise UserError(msg)

            base_currency_rate = rate_info[0]
            for currency, (rate, date_rate) in parsed_data.items():
                rate_value = rate / base_currency_rate

                currency_id = currency_id_by_name.get(currency, None)
                if currency_id:  # if rate provider base currency is not active, it will be present in parsed_data
                    already_existing_rate = currency_rates.get((currency_id.id, date_rate, company.id), False)
                    if already_existing_rate:
                        already_existing_rate.rate = rate_value
                    else:
                        CurrencyRate.create({
                            'currency_id': currency_id.id,
                            'rate': rate_value,
                            'name': date_rate,
                            'company_id': company.id,
                        })
